module HaPar
 (
  dataDeclsFromModule,
  getDataDecls,
  readHaFile
 )
where

import Language.Haskell.Parser
import Language.Haskell.Syntax


import TDecl


{-
  Funo que l um ficheiro Haskell e extrai a informao sobre as
  declaraes de tipos.
-}
readHaFile :: FilePath -> IO [TDecl]
readHaFile f = do --IO
		 texto <- readFile f
		 let parseRes = parseModule texto
	         case parseRes of
		   (ParseOk hsmodule) -> return (getDataDecls hsmodule)
		   _ -> putStrLn "Erro de parsing..." >> return []



{-
  Funo que extrai a informao das declaraes de tipo contidas num texto
  com cdigo de um mdulo Haskell.
-}
dataDeclsFromModule :: String -> [TDecl]
dataDeclsFromModule modulo = case parseModule modulo of
			       (ParseOk hsmodule) -> getDataDecls hsmodule
			       _ -> error "Erro de parsing..."

{-
  Funo que extrai a informao das declaraes de tipo contidas num mdulo
  Haskell (j deve ter sido executado o Parser...)
-}
getDataDecls :: HsModule -> [TDecl]
getDataDecls (HsModule _ _ _ _ decls) = getDDecls decls
  where
    getDDecls [] = []
    getDDecls (HsDataDecl _ c t v cns _:xs) = (p c t v cns):(getDDecls xs)
    getDDecls (HsNewTypeDecl _ c t v cn _:xs) = (p c t v [cn]):(getDDecls xs)
    getDataDecls (_:xs) = (getDDecls xs)
    p ctxt tipo vars constrs = MkTD {tDesc = getId tipo,
				     tArgs = map getId vars,
				     tConsts = map procC constrs}
    procC (HsRecDecl _ _ _) = error "Records no permitidos..."
    procC (HsConDecl _ c l) = MkCD {cDesc = (show c),
				    cArgs = map procCArg l}
    procCArg (HsBangedTy t) = error "Banged types no permitidos!"
    procCArg (HsUnBangedTy t) = convTy t



getId (HsIdent s ) = s
getId _ = error "No permitido..."


getQId (UnQual n) = getId n
getQId (Special HsUnitCon) = error "UNIT"
getQId (Special HsListCon) = error "LIST"
getQId (Special HsFunCon) = error "FUN"
getQId _ = error "Qualified names no suportados!"


convTy (HsTyFun x y) = TArrow (convTy x) (convTy y)
convTy (HsTyTuple l) = TTuple (map convTy l)
convTy (HsTyCon (Special HsUnitCon)) = TUnit
convTy (HsTyCon c) = TFApp (getQId c) []
convTy (HsTyApp (HsTyCon (Special HsListCon)) t) = TList (convTy t)
convTy (HsTyApp x y) = case convTy x of
		         (TFApp c l) -> TFApp c (l++[(convTy y)])
			 _ -> error "TyApp no suportado!!!"
convTy (HsTyVar v) = TVar (getId v)



------------------------------------------------------------------------
------------------------------------------------------------------------
--                                                                    --
--  EXEMPLOS...                                                       --
--                                                                    --
------------------------------------------------------------------------
------------------------------------------------------------------------


exModule = unlines ["module Datas where",
		    "data Boo = True | False deriving (Eq,Show)",
		    "data L a = Nil | Cons (a,L a) deriving Eq",
		    "data FT a b = L [a] | N b (FT a b) (FT a b)"]

exDecls = dataDeclsFromModule exModule
