module Smonad where

import Mpi

data StTra s a = T { tranf :: (s -> (a, s)) }

-- cf. com data Func  s a = F { func  ::  s -> a  }


instance Monad (StTra s) where
         return a    = T (\s -> (a,s))
         (T t) >>= f = T (ap . ( (tranf . f) >< id) . t )
{-- ie:
         (T f) >>= g = T (\s -> let (a,s') = f s
                                    T k    = g a
                                in k s')
--}

instance Functor (StTra s) where
         fmap f t = do { a <- t ; return (f a) }
-- ie:   fmap f (T g) = T(\s -> let (a,s') = g s in (f a,s'))


-- program execution

exec (T prog) s = prog s

-- generic state transformers 

read   :: StTra a a
read = T(split id id)

write :: a  -> StTra a a
write a = update (const a)

update :: (a -> a) -> StTra a a
update f = T(split id f)

query :: (a -> b) -> StTra a b
query f = T(split f id)

trans :: (a -> a) -> (a -> b) -> StTra a b
-- a simple transation
trans g f = do { update g ; query f }

itrans :: (a -> b -> b) -> (a -> b -> c) -> a -> StTra b c
-- a transation with input
itrans g f a = do { update (g a) ; query (f a)}

