module TDecl where


{-
  Tipo de dados para conter a informao de uma declarao de tipo.
-}
data TDecl = MkTD { tDesc :: String,      -- Nome do tipo
                    tArgs :: [String],    -- Argumentos (variveis de tipo)
                    tConsts :: [CDecl]    -- Construtores
                  } deriving Eq

{-
  Tipo de dados para conter a informao de um construtor.
-}
data CDecl = MkCD { cDesc :: String,      -- Nome do construtor
                    cArgs :: [Type]       -- Tipo dos argumentos
                  } deriving Eq

{-
  Tipo de dados para armazenar um tipo Haskell
-}
data Type = TUnit                  -- Tipo ()
          | TVar String            -- Varivel de tipo
          | TList Type             -- Listas
          | TTuple [Type]          -- Tuplos
          | TArrow Type Type       -- Espao de funes (no considerar...)
          | TFApp String [Type]    -- Aplicao de um construtor de tipo
            deriving Eq


---------------------------------
--                             --
--  Instncias da classe Show  --
--                             --
---------------------------------

showListSep :: Show a => String -> [a] -> ShowS
showListSep s [] = id
showListSep s [t] = shows t
showListSep s (x:xs) = shows x . showString s . showListSep s xs


instance Show TDecl where
  showsPrec _ t = showString "data " . showString (tDesc t) . showChar ' '.
		  showString (foldr (\x r->x++" "++r) "" (tArgs t)) .
		  showString " = " .
		  showListSep " | " (tConsts t)


instance Show CDecl where
  showsPrec _ c = showString (cDesc c) . showListSep " " (cArgs c)


instance Show Type where
  showsPrec _ (TUnit) = showString "()"
  showsPrec _ (TVar v) = showString v
  showsPrec _ (TList t) = showString " [" . shows t . showChar ']'
  showsPrec _ (TTuple l) = showString " (" . showListSep "," l . showChar ')'
  showsPrec _ (TFApp f l) = showString " (" . showString f . showChar ' ' . 
                            showListSep " " l . showChar ')'
  showsPrec _ (TArrow t1 t2) = showString " (" . shows t1 . showString "->" .
			       shows t2 . showChar ')'


--------------------------------------------------------------------------

{-
   Exemplos:
   =========
-}
cNil = MkCD { cDesc = "Nil",
	      cArgs = []
	    }
cCons = MkCD { cDesc = "Cons",
	       cArgs = [(TTuple [TVar "a",TFApp "RList" [(TVar "a")]])]
	     }
tRList = MkTD { tDesc = "List",
		tArgs = ["a"],
		tConsts = [cNil,cCons]
	      }


--------------------

	      
