\documentclass[a4paper,11pt,portuges]{article}

\usepackage[latin1]{inputenc}
\usepackage{babel}
\usepackage{amsmath}
\usepackage{xypic}
\usepackage{a4wide}
\usepackage{mpi0304}


\begin{document}

\title{Trabalho Prtico n$$1}
\author{Mtodos de  Programao I}
\date{2003/2004}

\maketitle

\begin{abstract}
%  Este  o enunciado do primeiro trabalho prtico da disciplina de
%  Mtodos de Programao I (ano lectivo de 2003/2004). 
  O objectivo deste trabalho  realizar um pr-processador para
  programas na linguagem \textsl{Haskell} por forma a que sejam
  gerados de forma automtica as funes \texttt{cata}, \texttt{ana} e
  \texttt{hylo} para tipos de dados arbitrrios definidos pelo
  utilizador.
%Trata-se assim de uma oportunidade para maturar os
%  conceitos tericos estudados neste curso num 
\end{abstract}


\section{Prembulo}

Este trabalho deve ser realizado por grupos com um mximo de trs alunos,
e deve ser submetido de acordo com as instrues divulgadas na
pgina da disciplina antes da data limite a mencionada.

\section{Introduo}

\begin{code}
module List where

import Mpi
\end{code}

Considere-se a declarao de tipo \texttt{List} (cfr. \texttt{RList} da bblioteca \texttt{mpi.hs}) 
\begin{code}
data List a = Nil () | Cons (a,List a)
\end{code}
O significado deste tipo de dados determina que exista um isomorfismo
entre os conjuntos:
\[ \text{List a} \cong 1 + a  (\text{List a}) \]
sendo o par de funes que testemunha esse isomorfismo
\begin{eqnarray*}
  \mathrm{inList} &:& 1 + a  (\text{List a}) \to \text{List a} \\
%  \mathrm{inList} &=& [ Nil, Cons] \\
  \mathrm{outList} &:& \text{List a} \to 1 + a  (\text{List a}) %\\
%  \text{outList (Nil ())} &=& i_1\ () \\
%  \text{outList (Cons (x,l))} &=& i_2\ (x,l) \\
\end{eqnarray*}

Tambm associados a este tipo de dados  possvel identificar
``padres de recursividade'' que nos permitem definir funes que
manipulem valores desse tipo. Tal como o \texttt{foldr} captura a
essncia de um padro de recursividade sobre as listas primitivas do
\textsl{Haskell}, podemos considerar o que designamos por
\emph{catamorfismos} que percorrem toda a estrutura do tipo de dados
definido. Em diagrama:

\newcommand{\cata}[1]{(\![ #1 ]\!)}

\[
\xymatrix{
\mathrm{List\ a} \ar[d]_{\cata{g}} \ar[rr]^{\mathrm{outList}} & & 1 +
a \mathrm{List\ a} \ar[d]^{id + id  \cata{g}}\\
X &  & 1 + a  X \ar[ll]_{g} 
}
\]

De forma dual definem-se os \emph{anamorfismos} e, como a composio
de um catamorfismo com um anamorfismo, os \emph{hilomorfismos}.

\ 

Em resumo, da declarao de um tipo dados (como \texttt{List}
apresentado atrs) podemos derivar um conjunto de combinadores que nos
permitiro definir funes que manipulem valores desse tipo,
nomeadamente:
\begin{code}
inList :: Either () (a,List a) -> List a
--inList = either Nil Cons
inList (Left ()) = Nil ()
inList (Right (x,l)) = Cons (x,l)

outList :: List a -> Either () (a,List a)
outList (Nil ()) = Left ()
outList (Cons (x,l)) = Right (x,l)

recList :: (b->c) -> Either () (a,b) -> Either () (a,c)
recList f = id -|- id >< f

cataList :: (Either () (a,b) -> b) -> List a -> b
cataList f = f . recList (cataList f) . outList

anaList :: (b -> Either () (a,b)) -> b -> List a
anaList f = inList . recList (anaList f) . f

hyloList :: (Either () (a,b) -> b) ->
            (c -> Either () (a,c)) -> c -> b
hyloList f g = cataList f . anaList g
\end{code}


Uma vez definidos estes combinadores, estamos habilitados a definir
funes como as que tem vindo a ser estudadas neste curso. Por
exemplo:

\begin{code}
fact = hyloList f g
  where f = either (const 1) (uncurry (*))
        g 0 = Left ()
        g n = Right (n,n-1)
\end{code}


\section{Trabalho a Realizar}

Para um tipo de dados \texttt{T} lido de um ficheiro, pretende-se
gerar automaticamente as funes \texttt{inT}, \texttt{outT},
\texttt{recT}, \texttt{cataT}, \texttt{anaT} e \texttt{hyloT}.

\ 

Para a tarefa de ler um ficheiro com cdigo \textsl{Haskell} e extrair
da a informao das declaraes de tipos  fornecido um ficheiro que
utiliza bibliotecas disponibilizadas pelo GHC (ver adiante). Assim, a
funcionalidade que dever ser implementada neste trabalho consiste em
gerar as funes referidas a partir da informao armazenada em
estruturas de dados adequadas.

As declaraes de tipo devero ser representadas utilizando os
seguintes tipos \textsl{Haskell}:

\begin{code}
-- Tipo de dados para conter a informao de uma declarao de tipo.
data TDecl = MkTD { tDesc :: String,      -- Nome do tipo
                    tArgs :: [String],    -- Argumentos (variveis de tipo)
                    tConsts :: [CDecl]    -- Construtores
                  } deriving Eq
\end{code}
\begin{code}
-- Tipo de dados para conter a informao de um construtor.
data CDecl = MkCD { cDesc :: String,      -- Nome do construtor
                    cArgs :: [Type]       -- Tipo dos argumentos
                  } deriving (Eq,Show)
\end{code}
\begin{code}
-- Tipo de dados para armazenar um tipo Haskell
data Type = TUnit                  -- Tipo ()
          | TVar String            -- Varivel de tipo
          | TList Type             -- Listas
          | TTuple [Type]          -- Tuplos
          | TArrow Type Type       -- Espao de funes (no considerar...)
          | TFApp String [Type]    -- Aplicao de um construtor de tipo
            deriving (Eq,Show) 
\end{code}

A ttulo de exemplo, a declarao de tipo \texttt{List} 
 representada pelo valor:
\begin{code}
tList = MkTD { tDesc = "List",
               tArgs = ["a"],
               tConsts = [cNil,cCons]
              }
  where cNil = MkCD { cDesc = "Nil",
                      cArgs = [TUnit]
                    }
        cCons = MkCD { cDesc = "Cons",
                       cArgs = [(TTuple [TVar "a",TFApp "List" [(TVar "a")]])]
                       }
\end{code}



Numa primeira fase,  conveniente considerar algumas simplificaes
sobre a forma dos tipos que iro ser processados:

\begin{itemize}
\item Todos os construtores dispem de um e um s tipo de dados como
  argumento.
\item Os tipos de dados no fazem referncia a quaisquer outros
  construtores de tipos (e.g. no fazem uso de listas, ou de outros
  tipos definidos pelo utilizador).
\end{itemize}

\section{Material fornecido como kit para o projecto}
Entre na pgina da disciplina e descarregue,
descomprimindo-o (e.g.\ via \texttt{unzip}), o ficheiro
{\texttt{mpi0304mp.zip}} que contm o respectivo material pedaggico.
%
Para alm de outros ficheiros relevantes para a disciplina, dever obter:
\begin{enumerate}
\item	\texttt{mpi0304t1.lhs} - trata-se do ficheiro que est a ler neste
momento, escrito em \emph{literate} \textsl{Haskell}. Isto significa que:
	\begin{itemize}
	\item	se o carregar no \textsl{Hugs} ou \textsl{GHCi},
		o interpretador carregar o cdigo
		\textsl{Haskell} nele contido e interpret-lo-.
	\item	se o processar via \LaTeX\ (ou PDF\LaTeX) obter este mesmo
		documento em \textsl{PDF}.
		Sugesto: experimente
\begin{verbatim}
       latex mpi0304t1.lhs
       dvips -o mpi0304t1.ps mpi0304t1
       ps2pdf mpi0304t1.ps
\end{verbatim}
		ou, mais simplesmente, 
		\texttt{pdflatex mpi0304t1.lhs}
		\footnote{
		Se no est habituado a \LaTeX: em \textsl{Linux}, faz parte da
		distribuio standard e  s experimentar;
		em \textsl{Windows}, sugere-se a instalao de \textsl{MiK\TeX}
		(\texttt{http://www.miktex.org/}).}.
	\end{itemize}

\item
Com vista a completar a funcionalidade do projecto fornece-se cdigo
\textsl{Haskell} para extrair de um mdulo as declaraes de tipo a
contidas. Mais precisamente, so disponibilizados os ficheiros:

\begin{description}
\item[\texttt{HaPar.hs}:] Faz uso do parser do \textsl{Haskell}
  (incluido na verso $6.01$ do GHC) para extrair as declaraes de
  tipo de um ficheiro contendo um mdulo Haskell. Requer a opo
  \texttt{-package haskell-src}.
\item[\texttt{TDecl.hs}:] Define os tipos de dados referidos atrs
  (\texttt{TDecl}, \texttt{CDecl} e \texttt{Type}) juntamente com
  algumas funes para visualizao de valores desses tipos
  (instncias da classe \texttt{Show}).
\end{description}

O primeiro destes ficheiros requer a utilizao do GHC. No entanto, a
sua utilizao no interfere com o desenvolvimento da funcionalidade
pedida.
\end{enumerate}

\section{Valorizaes}

Alguns pontos sugeridos para melhorar o trabalho.

\subsection{Admitir construtores sem argumentos}

Em \textsl{Haskell} so admitidos construtores sem argumentos. Neste
trabalho, essa possibilidade  facilmente incorporada ao
identificarmos um construtor sem argumentos com um construtor com
argumento de tipo \texttt{()} na construo das funes \texttt{in} e
\texttt{out} (ver exemplo no ponto seguinte).


\subsection{Admitir construtores \emph{curried}}

Outra prtica comum em \textsl{Haskell}  declarar os construtores com
mltiplos argumentos na sua verso \emph{curried}. De facto, a
declarao do tipo de listas mais natural para um programador
\textsl{Haskell} seria:

\begin{code}
data List' a = Nil' | Cons' a (List' a)
\end{code}

Mais uma vez, facilmente se ajustam as funes \texttt{in} e
\texttt{out} por forma a acomodar esta diferena. Ilustrando para o
caso do \texttt{outList'} teramos:
\begin{code}
outList' :: List' a -> Either () (a,List' a)
outList' Nil' = Left ()
outList' (Cons' x l) = Right (x,l)
\end{code}

\subsection{Suportar a utilizao de listas nos argumentos de
  construtores}

Ocasionalmente, temos necessidade de considerar tipos mais complexos
nos argumentos dos construtores, como as listas primitivas do
\textsl{Haskell}. Um exemplo  o tipo \emph{Rose Trees} j apresentado
nas aulas terico-prticas.

\begin{code}
data RoseT a = Vazia | Node a [RoseT a]
\end{code}

A utilizao das listas manifesta-se na utilizao do
\texttt{map}. Ilustrando no diagrama do catamorfismo, temos:

\[
\xymatrix{
\mathrm{RoseT\ a} \ar[d]_{\cata{g}} \ar[rr]^{\mathrm{outList}\quad} & & 1 +
a  [\mathrm{RoseT\ a}] \ar[d]^{id + id  (map\ \cata{g})}\\
X &  & 1 + a  [X] \ar[ll]_{g} 
}
\]

Permitir a utilizao deste tipo de declaraes passa ento
por ajustar convenientemente a definio da funo \texttt{recT}.

\subsection{Suportar tipos de dados dependentes de outros tipos}


Uma generalizao do problema referido no ponto anterior consiste em
considerar tipos de dados que dependam de um outro tipo de dados
fornecido pelo utilizador (e.g. definir as \emph{Rose Trees} utilizando
o tipo \texttt{List} apresentado atrs). A estratgia seguida no caso
das listas primitivas pode ser generalizada para cobrir tambm este
caso.



%\subsection{Tipos de dados mutuamente recursivos}




%\section{Uma Viso Alternativa (valorizao)}
%
%\textsf{%
%\textbf{Alerta:} % 
%
%Esta seco destina-se a quem pretender aprofundar 
%Os conceitos so apresentados numa forma (ainda) mais abstracta. Como
%tal, dever ser sensato ignorar a formulao aqui apresentada numa
%fase em que esses conceitos ainda no se encontrem devidamente
%solidificados (sob risco de contribuirem mais para a sua baralhao do
%que para uma compreenso mais aprofundada). De qualquer forma, uma
%leitura no dever causar dano\ldots }




%\[
%\xymatrix{
%X \ar[d]_{\cata{g}} \ar[rr]^{\mathrm{outList}} & & F\ X\ar[d]^{F\  \cata{g}}\\
%Y &  & F\ Y \ar[ll]_{g} 
%}
%\]




\end{document}
