\documentclass[12pt]{article}
\usepackage{fancyvrb}
\usepackage{graphicx}

\DefineVerbatimEnvironment{code}{Verbatim}{frame=single,framesep=2mm,framerule=0.5pt} 


\usepackage[latin1]{inputenc}



\textwidth = 6.5 in
\textheight = 9 in
\oddsidemargin = 0.0 in
\evensidemargin = 0.0 in
\topmargin = 0.0 in
\headheight = 0.0 in
\headsep = 0.0 in
\parskip = 0.2in
\parindent = 0.0in

\newtheorem{theorem}{Theorem}
\newtheorem{corollary}[theorem]{Corollary}
\newtheorem{definition}{Definition}

\author{Mtodos de Programao I -- 2003/04}
\title{Trabalho Prtico n. 2\\ Programao com \emph{Quadtrees}}
\date{}

\begin{document}
\maketitle

\section*{Prembulo}

Este trabalho prtico  constitudo por duas seces. A primeira contm questes obrigatrias, e a segunda questes para valorizao. Sempre que possvel, dever escrever as funes pedidas utilizando os padres de recursividade (anamorfismos, catamorfismos, hilomorfismos) adequados. 

Considere como origem das coordenadas numa imagem rectangular o seu vrtice superior esquerdo.

\section*{Introduo}

Uma \emph{quadtree}  simplesmente uma rvore quadrtica etiquetada nas folhas, mas  contendo normalmente (no contexto que consideraremos) tambm informao nos ns. Um tipo de aplicao muito em voga destas rvores concerne a representao de informao espacial (por exemplo no domnio dos Sistemas de Informao Geogrfica, mas tambm nos motores de animao utilizados em jogos 3D de ltima gerao). 

Por informao de natureza espacial entende-se uma distribuio de um conjunto de objectos numa determinada representao de um \emph{terreno} (por exemplo)
Um dos mtodos possiveis para a representao deste tipo de informao utiliza uma diviso recursiva do terreno em sub-terrenos, diviso esta que pode ser representada por uma rvore.

Neste trabalho prtico ocupar-nos-emos com a aplicao de \emph{quadtrees} na representao e processamento de imagens.


\section*{I -- \emph{Quadtrees} na representao de imagens}

Existem dois mtodos essenciais para a representao de imagens a cores:

\begin{enumerate}

\item Como \emph{bitmaps}, matrizes bidimensionais contendo em cada posio informao relativa um pixel (poder ser um booleano (bit) caso se trate de uma imagem a preto e branco; um valor inteiro caso se trate de uma imagem em \emph{grayscale} ou uma lista de 3 inteiros no caso das imagens a cores).

\item Como \emph{mapas vectoriais}, sendo ento cada imagem descrita por uma lista dos objectos que a constituem (por exemplo, quadrado de cor azul de lado 30 e coordenadas (20, 10) para o vrtice inferior esquerdo).

\end{enumerate}

As \emph{quadtrees} proporcionam um terceiro mtodo: uma optimizao dos \emph{bitmaps} em que se representa de forma atmica informao relativa a todo um bloco (rectangular) de pixels. A ideia  considerar a imagem (que se considera ter uma forma rectangular) dividida em quatro quadrantes iguais; cada um destes  em si uma imagem rectangular que se divide de igual forma. A imagem assim dividida corresponde a um n da \emph{quadtree}, em que se armazena informao geomtrica, nomeadamente as coordenadas de dois vrtices opostos. Os seus descendentes so (as rvores correspondentes a) os seus 4 quadrantes.

Quadrantes (ou \emph{seces}) constitudos apenas por pixels da mesma cor no precisam de ser divididos: basta represent-los por folhas da rvore, onde  guardada informao relativa  cor. Naturalmente, em imagens com grandes manchas da mesma cor,  possvel poupar espao considervel por comparao com a representao por \emph{bitmap}. No caso limite, a seco mais pequena  um pixel.


\subsection*{Tarefas} 

Pretende-se escrever as seguintes funes:

\begin{enumerate}

\item (\verb+mkQTree :: BitMap -> QTree+) criao de uma \emph{quadtree} a partir de um \emph{bitmap} fornecido

\item (\verb+scale :: Int -> QTree -> QTree+) operao de ampliao de uma imagem, segundo o factor de multiplicao fornecido. Esta funo poder ser generalizada para admitir redues de imagens (factor negativo) e ainda factores de ampliao fraccionais (por exemplo 1.5 ampliar a imagem aumentando ambos os seus lados em 50\%).

\item (\verb+mirrorV, mirrorH :: QTree -> QTree+) operaes de rebatimento vertical e horizontal

\item (\verb+rotateD, rotateR :: QTree -> QTree+) operaes de rotao de 90 graus nos dois sentidos

\item (\verb+mapColour :: (Colour -> Colour) -> QTree -> QTree+) mapeamento uniforme de cor em toda a imagem


\end{enumerate}


\section*{II -- Sobreposio de Imagens e Utilizao de Informao Vectorial}

Escreva agora as seguintes funes:

\begin{enumerate}

  \item (\verb+sp :: QTree -> QTree -> QTree+)  operao de sobreposio de rvores. Dever alterar o tipo de dados para lidar com a noo de cor transparente (i.e. uma cor visualizada como branco, mas que quando sobreposta a outra permite ``ver'' a cor subjacente). 
  
  \item Funes para desenhar formas geomtricas sobre uma imagem. Dever ser possvel, por exemplo, a seguinte invocao:
  
\begin{verbatim}
drawRectangle r pos t
\end{verbatim}
  
que permite acrescentar  \emph{quadtree} \verb+t+  o rectngulo \verb+r+, sendo o seu vrtice superior esquerdo colocada nas coordenadas dadas por \verb+pos+. 

Comece por considerar apenas o caso dos rectngulos com lados paralelos s margens da imagem; para isso defina um tipo adequado (contendo as dimenses e cor do rectngulo). 

A estrutura das \emph{quadtrees} permite efectuar esta operao de forma muito optimizada; dever ter isto em conta na funo que escrever. Em seguia poder generalizar a funo para lidar com outras formas geomtricas.
  
\end{enumerate}

\section*{Tipos de Dados a Utilizar}

\begin{itemize}

\item Tipo de dados a utilizar para a representao de \emph{bitmaps}:

\begin{code}
type Comp = Int
type Colour = (Comp, Comp, Comp)
data Pixel = Pix Colour
data BitMap = Bmp (Int, Int, [[Pixel]])
\end{code}
As componentes (R, G, B) de uma cor so inteiros com valor entre 0 e 255. Num \emph{bitmap} \verb+Bmp(r,c,l)+, \verb+l+  uma lista de comprimento \verb+r+, contendo listas de 
comprimento \verb+c+. Cada uma destas listas representa uma linha da imagem.

\item Tipo de dados \emph{Quadtree}

\begin{code}
type Point = (Int, Int)
type Coords = (Point, Point)
type Section = (Coords, Colour)
data QTree = Empty | Leaf Section | Node Coords QTree QTree QTree QTree
\end{code}
A cada n e cada seco (folha) de uma rvore esto associadas duas coordenadas, dos vrtices \emph{superior esquerdo} e \emph{inferior direito}. Observe-se que a presena do constructor \verb+Empty+ deve-se  eventual necessidade de dividir imagens com apenas dois pixels em quadrantes; neste caso dois dos quadrantes sero vazios.

\end{itemize}


\section*{O Kit Fornecido}

Os \emph{bitmaps} sero armazenados em ficheiros com a seguinte estrutura: 

\begin{itemize}

\item uma linha contendo um inteiro correspondente ao nmero $r$ de linhas da imagem, seguida de
\item uma linha contendo um inteiro correspondente ao nmero $c$ de colunas da imagem, seguida de
\item $r \times c$ linhas, cada uma contendo trs inteiros separados por espaos. Estes trs nmeros codificam uma cor segundo o pradro RGB. Cada uma destas linhas de texto corresponde ento a um pixel; os pixels aparecem no ficheiro a partir do vrtice superior esquerdo da imagem, sendo depois varridas as linhas uma a uma da esquerda para a direita.

\end{itemize}

O ficheiro \verb+visual.NET.zip+ contm um visualizador de bitmaps (escrito sobre a plataforma .NET) descritos segundo este formato. 
Para poder ler e visualizar bitmaps, ter antes de mais que

\begin{enumerate}
\item Escrever uma funo mondica de leitura de bitmaps para o tipo \verb+BitMap+.
\item Escrever uma funo \verb+show+ para o tipo \verb+BitMap+, que escreva ficheiros segundo o formato descrito.
\end{enumerate}


\section*{Exemplo de Diviso de uma Imagem em Quadrantes}

A imagem total corresponde ao n numerado com 1, a raz da rvore. A sequncia 2,6,10.14 representa um caminho na rvore, desde a raz at uma folha correspondente a um pixel.No entanto a construo da rvore poder parar mais cedo, caso se atinja uma seco em que todos os pixels tenham a mesma cor (cores no se encontram representadas na figura).

\vspace{.2in}
\centerline {
\includegraphics[width=2in]{Picture2}
\includegraphics[width=2in]{Picture3}
}

\centerline{ 
\includegraphics[width=2in]{Picture4}
\includegraphics[width=2in]{Picture5}
}
\vspace{.2in}


 \end{document} 
