
data Tree a = Leaf a | Node (Tree a) (Tree a)

  deriving Show

-- funcao label na sua versao no mondica seria:
-- label :: Tree a -> Int -> (Int, Tree Int)
-- label t :: TEstado -> (TEstado, TResult)
-- label mondica:
-- label :: Tree a -> Estado (Tree Int)


-- N.B: O tipo do estado e' Int; o polimorfismo (var. de tipo a) diz
-- respeito apenas aos valores monadicos lidos a partir do estado.

data Estado a = E (Int -> (Int, a))

instance Monad Estado where
-- return :: a -> Estado a
  return x = E(\n -> (n,x))

-- (>>=) :: Estado a -> (a -> Estado b) -> Estado b
  (E e) >>= f = E(\n -> let (n',x) = e n
                            E e' = f x
                        in e' n')


-- Funcao de consulta e actualizacao do Estado
-- o argumento da funcao e' o estado antigo
-- a primeira componente do par e' o novo valor do estado
-- a segunda componente e' o valor (que pode ser de qualquer tipo) 
-- obtido ao fazer "x <- getLabel"; neste caso devolve-se 
-- simplesmente o valor antigo do estado 

getLabel :: Estado Int
getLabel = E(\n -> (n+1, n))


-- verso mondica de 'label' recebe input e devolve output mondico 
-- isto significa apenas que no calculo do resultado ha um efeito lateral
-- de alteracao do estado!

lab :: Tree a -> Estado (Tree Int)
lab (Leaf x) = do n <- getLabel
                  return (Leaf n)
lab (Node esq dir) = do esq' <- lab esq
                        dir' <- lab dir
                        return (Node esq' dir')

--esta primeira clausula tambem funciona. Porque?
--lab (Leaf x) = E(\n -> (n+1, Leaf(n)))


-- interface para inicializar estado e retirar monad do output
label :: Tree a -> Tree Int
label t = let E e = lab t
          in snd (e 100)


-- funcao de teste
test = let t = Node (Leaf 'a') (Leaf 'b')
       in label (Node t t)

